@@ -80,8 +80,24 @@ enum Command {
     Record {
         #[structopt(short, long, default_value = "5")]
         interval_s: u64,
+        /// Store retention in seconds. Data is stored in 24 hour shards.
+        /// Whever an entire shard of data is outside the retention period it
+        /// is discarded. That is, any data older than retention + 24 hours
+        /// is guaranteed to be discarded.
+        ///
+        /// N.B. If --store-size-limit is set, data may be discarded earlier
+        ///      than the specified retention.
         #[structopt(long)]
         retain_for_s: Option<u64>,
+        /// Store size limit in bytes. Data is stored in 24 hour shards.
+        /// Shards before the active shard are deleted, oldest first,
+        /// according to the size limit. Enforcement is only triggered on new
+        /// shard creation.
+        ///
+        /// N.B. Since the active shard cannot be deleted, the size limit may
+        ///      be exceeded by a single active shard.
+        #[structopt(long)]
+        store_size_limit: Option<u64>,
         /// Whether or not to collect io.stat for cgroups which could
         /// be expensive
         #[structopt(long)]
@@ -103,13 +119,6 @@ enum Command {
         /// You can expect up to ~4.5x smaller data files
         #[structopt(long)]
         compress: bool,
-        /// WARNING: This flag is ignored in open source. In open source, data
-        ///          is stored as CBOR. This flag is temporary and will be removed in
-        ///          the near future.
-        ///
-        /// Store data as CBOR. If unset, data is stored as Thrift.
-        #[structopt(long, hidden = true)]
-        use_cbor: bool,
     },
     /// Replay historical data (interactive)
     Replay {
@@ -270,6 +279,33 @@ fn check_for_exitstat_errors(logger: &slog::Logger, receiver: &Receiver<Error>)
     false
 }
 
+/// Discard old data shards in store according to store size limit and retention
+fn cleanup_store(
+    store: &store::StoreWriter,
+    logger: &slog::Logger,
+    store_size_limit: Option<u64>,
+    retention: Option<Duration>,
+) -> Result<()> {
+    if let Some(limit) = store_size_limit {
+        if !store
+            .try_discard_until_size(limit, logger.clone())
+            .context("Failed to discard earlier data")?
+        {
+            warn!(
+                logger,
+                "Failed to limit store size since the current shard is \
+                greater than the limit"
+            );
+        }
+    }
+    if let Some(retention) = retention {
+        store
+            .discard_earlier(SystemTime::now() - retention, logger.clone())
+            .context("Failed to discard earlier data")?;
+    }
+    Ok(())
+}
+
 /// Special Error that indicates the program should stop now. It represents an
 /// actual signal, e.g. SIGINT, SIGTERM, that is handled by below and thus below
 /// can shutdown gracefully.
@@ -431,13 +467,13 @@ fn real_main(init: init::InitToken) {
         Command::Record {
             ref interval_s,
             ref retain_for_s,
+            ref store_size_limit,
             ref collect_io_stat,
             ref port,
             ref skew_detection_threshold_ms,
             ref disable_disk_stat,
             ref disable_exitstats,
             ref compress,
-            ref use_cbor,
         } => {
             logutil::set_current_log_target(logutil::TargetLog::Term);
             run(
@@ -453,13 +489,13 @@ fn real_main(init: init::InitToken) {
                         Duration::from_secs(*interval_s as u64),
                         below_config,
                         retain_for_s.map(|r| Duration::from_secs(r as u64)),
+                        *store_size_limit,
                         *collect_io_stat,
                         Duration::from_millis(*skew_detection_threshold_ms),
                         debug,
                         *disable_disk_stat,
                         *disable_exitstats,
                         *compress,
-                        *use_cbor,
                     )
                 },
             )
@@ -585,14 +621,14 @@ fn record(
     errs: Receiver<Error>,
     interval: Duration,
     below_config: &BelowConfig,
-    retain: Option<Duration>,
+    retention: Option<Duration>,
+    store_size_limit: Option<u64>,
     collect_io_stat: bool,
     skew_detection_threshold: Duration,
     debug: bool,
     disable_disk_stat: bool,
     disable_exitstats: bool,
     compress: bool,
-    use_cbor: bool,
 ) -> Result<()> {
     debug!(logger, "Starting up!");
 
@@ -600,15 +636,8 @@ fn record(
         bump_memlock_rlimit()?;
     }
 
-    // TODO(T92471373): Remove --use-cbor flag and hardcode format as CBOR.
-    // This is already the case for open source.
-    let format = if !cfg!(fbcode_build) || use_cbor {
-        store::Format::Cbor
-    } else {
-        store::Format::Thrift
-    };
-
-    let mut store = store::StoreWriter::new(&below_config.store_dir, compress, format)?;
+    let mut store =
+        store::StoreWriter::new(&below_config.store_dir, compress, store::Format::Cbor)?;
     let mut stats = statistics::Statistics::new();
 
     let (exit_buffer, bpf_errs) = if disable_exitstats {
@@ -650,6 +679,7 @@ fn record(
         let collect_instant = Instant::now();
 
         let collected_sample = model::collect_sample(
+            &below_config.cgroup_root,
             &exit_buffer,
             collect_io_stat,
             &logger,
@@ -673,12 +703,16 @@ fn record(
 
         match collected_sample {
             Ok(s) => {
-                if let Err(e) = store.put(
+                match store.put(
                     post_collect_sys_time,
                     &DataFrame { sample: s },
                     logger.clone(),
                 ) {
-                    error!(logger, "{:#}", e);
+                    Ok(/* new shard */ true) => {
+                        cleanup_store(&store, &logger, store_size_limit, /* retention */ None)?
+                    }
+                    Ok(/* new shard */ false) => {}
+                    Err(e) => error!(logger, "{:#}", e),
                 }
             }
             Err(e) => {
@@ -693,11 +727,9 @@ fn record(
             }
         };
 
-        if let Some(retention) = retain {
-            store
-                .discard_earlier(SystemTime::now() - retention, logger.clone())
-                .context("Failed to discard earlier data")?;
-        }
+        // Only check against retention and not size limit. Size limit is only
+        // checked on creation of successful write to a new shard.
+        cleanup_store(&store, &logger, /* store_size_limit */ None, retention)?;
 
         stats.report_store_size(below_config.store_dir.as_path());
 
@@ -735,7 +767,8 @@ fn live_local(
     let (exit_buffer, bpf_errs) = start_exitstat(logger.clone(), debug);
     let mut bpf_err_warned = false;
 
-    let mut collector = model::Collector::new(exit_buffer);
+    let mut collector =
+        model::Collector::new_with_cgroup_root(below_config.cgroup_root.clone(), exit_buffer);
     logutil::set_current_log_target(logutil::TargetLog::File);
     // Prepare advance obj for pause mode
     let mut adv = new_advance_local(
